@@ -42,6 +42,42 @@
 #include <sys/stat.h>
 #include <dirent.h>
 
+/******************************************************************************/
+/* A quick custom widget to ensure that the left handed toggle works no matter
+ * which button is pressed.
+ */
+typedef struct { GtkCheckButton parent; } MouseCappletCheckButton;
+typedef struct { GtkCheckButtonClass parent; } MouseCappletCheckButtonClass;
+GNOME_CLASS_BOILERPLATE (MouseCappletCheckButton, mouse_capplet_check_button,
+			 GtkCheckButton, GTK_TYPE_CHECK_BUTTON)
+static void mouse_capplet_check_button_instance_init (MouseCappletCheckButton *obj) { }
+
+static gboolean
+mouse_capplet_check_button_button_press (GtkWidget *widget, GdkEventButton *event)
+{
+	if (event->type == GDK_BUTTON_PRESS) {
+		if (!GTK_WIDGET_HAS_FOCUS (widget))
+			gtk_widget_grab_focus (widget);
+		gtk_button_pressed (GTK_BUTTON (widget));
+	}
+	return TRUE;
+}
+static gboolean
+mouse_capplet_check_button_button_release (GtkWidget *widget, GdkEventButton *event)
+{
+      gtk_button_released (GTK_BUTTON (widget));
+      return TRUE;
+}
+
+static void
+mouse_capplet_check_button_class_init (MouseCappletCheckButtonClass *klass)
+{
+	GtkWidgetClass *widget_class = (GtkWidgetClass *)klass;
+	widget_class->button_press_event   = mouse_capplet_check_button_button_press;
+	widget_class->button_release_event = mouse_capplet_check_button_button_release;
+}
+/******************************************************************************/
+
 enum
 {
 	DOUBLE_CLICK_TEST_OFF,
@@ -192,13 +228,6 @@ threshold_from_gconf (GConfPropertyEditor *peditor,
 	return new_value;
 }
 
-static GConfValue *
-threshold_to_gconf (GConfPropertyEditor *peditor, 
-		    const GConfValue *value)
-{
-	return gconf_value_float_to_int (value);
-}
-
 static GConfValue *
 drag_threshold_from_gconf (GConfPropertyEditor *peditor, 
 			   const GConfValue *value)
@@ -212,13 +241,6 @@ drag_threshold_from_gconf (GConfPropertyEditor *peditor,
 	return new_value;
 }
 
-static GConfValue *
-drag_threshold_to_gconf (GConfPropertyEditor *peditor, 
-		    const GConfValue *value)
-{
-	return gconf_value_float_to_int (value);
-}
-
 /* Retrieve legacy settings */
 
 static void
@@ -650,14 +672,14 @@ setup_dialog (GladeXML *dialog, GConfChangeSet *changeset)
 	gconf_peditor_new_numeric_range
 		(changeset, "/desktop/gnome/peripherals/mouse/motion_threshold", WID ("sensitivity_scale"),
 		 "conv-to-widget-cb", threshold_from_gconf,
-		 "conv-from-widget-cb", threshold_to_gconf,
+		 "conv-from-widget-cb", gconf_value_float_to_int,
 		 NULL);
 
 	/* DnD threshold */
 	gconf_peditor_new_numeric_range
 		(changeset, "/desktop/gnome/peripherals/mouse/drag_threshold", WID ("drag_threshold_scale"),
 		 "conv-to-widget-cb", drag_threshold_from_gconf,
-		 "conv-from-widget-cb", drag_threshold_to_gconf,
+		 "conv-from-widget-cb", gconf_value_float_to_int,
 		 NULL);
 
 	/* listen to cursors changing */
@@ -679,6 +701,9 @@ create_dialog (void)
 	GladeXML     *dialog;
 	GtkSizeGroup *size_group;
 
+	/* register the custom type */
+	(void) mouse_capplet_check_button_get_type ();
+
 	dialog = glade_xml_new (GNOMECC_DATA_DIR "/interfaces/gnome-mouse-properties.glade", "mouse_properties_dialog", NULL);
 	widget = glade_xml_get_widget (dialog, "prefs_widget");
 
@@ -755,6 +780,7 @@ main (int argc, char **argv)
 		g_signal_connect (dialog_win, "response",
 				  G_CALLBACK (dialog_response_cb), changeset);
 
+		capplet_set_icon (dialog_win, "mouse-capplet.png");
 		gtk_widget_show_all (dialog_win);
 
 		gtk_main ();
