@@ -11,16 +11,27 @@
 #include <libgnomevfs/gnome-vfs-async-ops.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 
+#include <libwindow-settings/gnome-wm-manager.h>
+
 #include "theme-common.h"
 #include "capplet-util.h"
 #include "activate-settings-daemon.h"
 #include "gconf-property-editor.h"
 #include "file-transfer-dialog.h"
 
-#define GTK_THEME_KEY "/desktop/gnome/interface/gtk_theme"
+/* FIXME: This shouldn't be hardcoded
+ */
+#define METACITY_THEME_LOCATION "/usr/share/themes"
+
+#define GTK_THEME_KEY     "/desktop/gnome/interface/gtk_theme"
+#define WINDOW_THEME_KEY  "/desktop/gnome/applications/window_manager/theme"
+#define METACITY_THEME_DIR "/apps/metacity/general"
+#define METACITY_THEME_KEY METACITY_THEME_DIR "/theme"
 
 #define MAX_ELEMENTS_BEFORE_SCROLLING 8
 
+static void read_themes (GladeXML *dialog);
+
 enum
 {
   THEME_NAME_COLUMN,
@@ -40,8 +51,12 @@ static GtkTargetEntry drop_types[] =
 };
 
 static gint n_drop_types = sizeof (drop_types) / sizeof (GtkTargetEntry);
+
 static gboolean setting_model = FALSE;
 static gboolean initial_scroll = TRUE;
+static gboolean window_setting_model = FALSE;
+static gboolean window_initial_scroll = TRUE;
+static gboolean running_theme_install = FALSE;
 
 static GladeXML *
 create_dialog (void)
@@ -55,7 +70,7 @@ create_dialog (void)
 
 static void
 theme_selection_changed (GtkTreeSelection *selection,
-			 gpointer          data)
+			 const gchar      *gconf_key)
 {
   GtkTreeModel *model;
   gchar *new_key;
@@ -83,25 +98,170 @@ theme_selection_changed (GtkTreeSelection *selection,
     {
       gchar *old_key;
 
-      old_key = gconf_client_get_string (client, GTK_THEME_KEY, NULL);
+      old_key = gconf_client_get_string (client, gconf_key, NULL);
       if (old_key && strcmp (old_key, new_key))
 	{
-	  gconf_client_set_string (client, GTK_THEME_KEY, new_key, NULL);
+	  gconf_client_set_string (client, gconf_key, new_key, NULL);
 	}
       g_free (old_key);
     }
   else
     {
-      gconf_client_unset (client, GTK_THEME_KEY, NULL);
+      gconf_client_unset (client, gconf_key, NULL);
     }
   g_free (new_key);
+  g_object_unref (client);
+}
+
+static void
+gtk_theme_selection_changed (GtkTreeSelection *selection,
+			     gpointer          data)
+{
+  theme_selection_changed (selection, GTK_THEME_KEY);
+}
+
+static void
+window_theme_selection_changed (GtkTreeSelection *selection,
+				gpointer          data)
+{
+  theme_selection_changed (selection, METACITY_THEME_KEY);
+}
+
+static void
+load_theme_names (GtkTreeView *tree_view,
+		  GList       *theme_list,
+		  char        *current_theme)
+{
+  GList *list;
+  GtkTreeModel *model;
+  GtkWidget *swindow;
+  gint i = 0;
+  gboolean current_theme_found = FALSE;
+  GtkTreeRowReference *row_ref = NULL;
+
+  swindow = GTK_WIDGET (tree_view)->parent;
+  model = gtk_tree_view_get_model (tree_view);
+
+  setting_model = TRUE;
+  gtk_list_store_clear (GTK_LIST_STORE (model));
+  
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swindow),
+				  GTK_POLICY_NEVER, GTK_POLICY_NEVER);
+  gtk_widget_set_usize (swindow, -1, -1);
+
+  for (list = theme_list; list; list = list->next)
+    {
+      const char *name = list->data;
+      GtkTreeIter iter;
+
+      gtk_list_store_prepend (GTK_LIST_STORE (model), &iter);
+      gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+			  THEME_NAME_COLUMN, name,
+			  -1);
+
+      if (strcmp (current_theme, name) == 0)
+	{
+	  GtkTreePath *path = gtk_tree_model_get_path (model, &iter);
+	  row_ref = gtk_tree_row_reference_new (model, path);
+	  gtk_tree_path_free (path);
+	  current_theme_found = TRUE;
+	}
+
+      if (i == MAX_ELEMENTS_BEFORE_SCROLLING)
+	{
+	  GtkRequisition rectangle;
+	  gtk_widget_size_request (GTK_WIDGET (tree_view), &rectangle);
+	  gtk_widget_set_usize (swindow, -1, rectangle.height);
+	  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (swindow),
+					  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
+	}
+      i++;
+    }
+
+  if (! current_theme_found)
+    {
+      GtkTreeIter iter;
+      GtkTreePath *path;
+
+      gtk_list_store_prepend (GTK_LIST_STORE (model), &iter);
+      gtk_list_store_set (GTK_LIST_STORE (model), &iter,
+			  THEME_NAME_COLUMN, current_theme,
+			  -1);
+
+      path = gtk_tree_model_get_path (model, &iter);
+      row_ref = gtk_tree_row_reference_new (model, path);
+      gtk_tree_path_free (path);
+    }
+
+  if (row_ref)
+    {
+      GtkTreePath *path;
+
+      path = gtk_tree_row_reference_get_path (row_ref);
+      gtk_tree_view_set_cursor (tree_view,path, NULL, FALSE);
+
+      if (initial_scroll)
+	{
+	  gtk_tree_view_scroll_to_cell (tree_view, path, NULL, TRUE, 0.5, 0.0);
+	  initial_scroll = FALSE;
+	}
+      
+      gtk_tree_path_free (path);
+      gtk_tree_row_reference_free (row_ref);
+    }
+  setting_model = FALSE;
+
+  g_free (current_theme);
 }
 
 static void
 read_themes (GladeXML *dialog)
 {
+  GList *theme_list, *gtk_theme_list, *metacity_theme_list;
+  GList *list;
   GConfClient *client;
-  GList *gtk_theme_list;
+  char *current_theme;
+
+  client = gconf_client_get_default ();
+
+  /* Read in themes
+   */
+  theme_list = theme_common_get_list ();
+
+  gtk_theme_list = NULL;
+  metacity_theme_list = NULL;
+  for (list = theme_list; list; list = list->next)
+    {
+      ThemeInfo *info = list->data;
+
+      if (info->has_gtk)
+	gtk_theme_list = g_list_prepend (gtk_theme_list, info->name);
+      if (info->has_metacity){
+	metacity_theme_list = g_list_prepend (metacity_theme_list, info->name);
+      }
+    }
+
+  current_theme = gconf_client_get_string (client, GTK_THEME_KEY, NULL);
+  if (current_theme == NULL)
+    current_theme = g_strdup ("Default");
+
+  load_theme_names (GTK_TREE_VIEW (WID ("theme_treeview")), gtk_theme_list, current_theme);
+  
+  current_theme = gconf_client_get_string (client, METACITY_THEME_KEY, NULL);
+  if (current_theme == NULL)
+    current_theme = g_strdup ("Default");
+
+  load_theme_names (GTK_TREE_VIEW (WID ("window_theme_treeview")), metacity_theme_list, current_theme);
+
+  g_list_free (gtk_theme_list);
+  g_list_free (metacity_theme_list);
+}
+
+static void
+window_read_themes (GladeXML *dialog)
+{
+  GConfClient *client;
+  GList *window_theme_list;
   GList *list;
   GtkTreeModel *model;
   GtkTreeView *tree_view;
@@ -109,43 +269,44 @@ read_themes (GladeXML *dialog)
   gint i = 0;
   gboolean current_theme_found = FALSE;
   GtkTreeRowReference *row_ref = NULL;
+  GnomeWindowManager *wm;
 
   client = gconf_client_get_default ();
 
-  gtk_theme_list = theme_common_get_list ();
-  tree_view = GTK_TREE_VIEW (WID ("theme_treeview"));
+  wm = gnome_wm_manager_get_current ();  
+  window_theme_list = gnome_window_manager_get_theme_list (wm);
+  g_object_unref (G_OBJECT (wm));
+
+  tree_view = GTK_TREE_VIEW (WID ("window_theme_treeview"));
   model = gtk_tree_view_get_model (tree_view);
 
-  setting_model = TRUE;
+  window_setting_model = TRUE;
   gtk_list_store_clear (GTK_LIST_STORE (model));
 
-  current_theme = gconf_client_get_string (client, GTK_THEME_KEY, NULL);
+  current_theme = gconf_client_get_string (client, WINDOW_THEME_KEY, NULL);
   if (current_theme == NULL)
     current_theme = g_strdup ("Default");
-  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (WID ("theme_swindow")),
+  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (WID ("window_theme_swindow")),
 				  GTK_POLICY_NEVER, GTK_POLICY_NEVER);
-  gtk_widget_set_usize (WID ("theme_swindow"), -1, -1);
+  gtk_widget_set_usize (WID ("window_theme_swindow"), -1, -1);
 
-  for (list = gtk_theme_list; list; list = list->next)
+  for (list = window_theme_list; list; list = list->next)
     {
-      ThemeInfo *info = list->data;
+      char *theme_name = list->data;
       GtkTreeIter iter;
 
-      if (! info->has_gtk)
-	continue;
-
       gtk_list_store_prepend (GTK_LIST_STORE (model), &iter);
       gtk_list_store_set (GTK_LIST_STORE (model), &iter,
-			  THEME_NAME_COLUMN, info->name,
+			  THEME_NAME_COLUMN, theme_name,
 			  -1);
 
-      if (strcmp (current_theme, info->name) == 0)
+      if (strcmp (current_theme, theme_name) == 0)
 	{
 	  GtkTreeSelection *selection;
 
 	  selection = gtk_tree_view_get_selection (tree_view);
 	  gtk_tree_selection_select_iter (selection, &iter);
-	  if (initial_scroll)
+	  if (window_initial_scroll)
 	    {
 	      GtkTreePath *path;
 
@@ -160,8 +321,8 @@ read_themes (GladeXML *dialog)
 	{
 	  GtkRequisition rectangle;
 	  gtk_widget_size_request (GTK_WIDGET (tree_view), &rectangle);
-	  gtk_widget_set_usize (WID ("theme_swindow"), -1, rectangle.height);
-	  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (WID ("theme_swindow")),
+	  gtk_widget_set_usize (WID ("window_theme_swindow"), -1, rectangle.height);
+	  gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (WID ("window_theme_swindow")),
 					  GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
 	}
       i++;
@@ -177,7 +338,7 @@ read_themes (GladeXML *dialog)
 			  THEME_NAME_COLUMN, current_theme,
 			  -1);
       gtk_tree_selection_select_iter (selection, &iter);
-      if (initial_scroll)
+      if (window_initial_scroll)
 	{
 	  GtkTreePath *path;
 
@@ -188,7 +349,7 @@ read_themes (GladeXML *dialog)
     }
 
 
-  if (row_ref && initial_scroll)
+  if (row_ref && window_initial_scroll)
     {
       GtkTreePath *path;
 
@@ -200,7 +361,7 @@ read_themes (GladeXML *dialog)
       gtk_tree_row_reference_free (row_ref);
       initial_scroll = FALSE;
     }
-  setting_model = FALSE;
+  window_setting_model = FALSE;
 
   g_free (current_theme);
 }
@@ -217,6 +378,29 @@ theme_key_changed (GConfClient *client,
   read_themes ((GladeXML *)user_data);
 }
 
+static void
+metacity_key_changed (GConfClient *client,
+		      guint        cnxn_id,
+		      GConfEntry  *entry,
+		      gpointer     user_data)
+{
+  if (strcmp (entry->key, METACITY_THEME_KEY))
+    return;
+
+  read_themes ((GladeXML *)user_data);
+}
+
+static void
+window_theme_key_changed (GConfClient *client,
+		   guint        cnxn_id,
+		   GConfEntry  *entry,
+		   gpointer     user_data)
+{
+  if (strcmp (entry->key, WINDOW_THEME_KEY))
+    return;
+
+  window_read_themes ((GladeXML *)user_data);
+}
 
 static void
 theme_changed_func (gpointer uri,
@@ -267,7 +451,7 @@ transfer_cancel_cb (GtkWidget *dlg, gchar *path)
  * see bugzilla.gnome.org #86141 for details
  */
 static gboolean
-transfer_done_idle_cb (gpointer data)
+transfer_done_targz_idle_cb (gpointer data)
 {
 	int status;
 	gchar *command;
@@ -284,12 +468,71 @@ transfer_done_idle_cb (gpointer data)
 	return FALSE;
 }
 
+static void
+window_show_manage_themes (GtkWidget *button, gpointer data)
+{
+	gchar *path, *command;
+	GnomeVFSURI *uri;
+	GnomeWindowManager *wm;
+	
+	wm = gnome_wm_manager_get_current ();
+	
+	path = gnome_window_manager_get_user_theme_folder (wm);
+	g_object_unref (G_OBJECT (wm));
+
+	uri = gnome_vfs_uri_new (path);
+
+	if (!gnome_vfs_uri_exists (uri)) {
+		/* Create the directory */
+		gnome_vfs_make_directory_for_uri (uri, 0775);
+	}
+	gnome_vfs_uri_unref (uri);
+
+
+	command = g_strdup_printf ("nautilus --no-desktop %s", path);
+	g_free (path);
+
+	g_spawn_command_line_async (command, NULL);
+	g_free (command);
+}
+
+
+/* this works around problems when doing fork/exec in a threaded app
+ * with some locks being held/waited on in different threads.
+ *
+ * we do the idle callback so that the async xfer has finished and
+ * cleaned up its vfs job.  otherwise it seems the slave thread gets
+ * woken up and it removes itself from the job queue before it is
+ * supposed to.  very strange.
+ *
+ * see bugzilla.gnome.org #86141 for details
+ */
+static gboolean
+transfer_done_tarbz2_idle_cb (gpointer data)
+{
+	int status;
+	gchar *command;
+	gchar *path = data;
+
+	/* this should be something more clever and nonblocking */
+	command = g_strdup_printf ("sh -c 'bzip2 -d -c < \"%s\" | tar xf - -C \"%s/.themes\"'",
+				   path, g_get_home_dir ());
+	if (g_spawn_command_line_sync (command, NULL, NULL, &status, NULL) && status == 0)
+		gnome_vfs_unlink (path);
+	g_free (command);
+	g_free (path);
+
+	return FALSE;
+}
+
 static void
 transfer_done_cb (GtkWidget *dlg, gchar *path)
 {
 	int len = strlen (path);
 	if (path && len > 7 && !strcmp (path + len - 7, ".tar.gz"))
-		g_idle_add (transfer_done_idle_cb, path);
+		g_idle_add (transfer_done_targz_idle_cb, path);
+	if (path && len > 8 && !strcmp (path + len - 8, ".tar.bz2"))
+		g_idle_add (transfer_done_tarbz2_idle_cb, path);
 	gtk_widget_destroy (dlg);
 }
 
@@ -349,23 +592,40 @@ install_dialog_response (GtkWidget *widget, int response_id, gpointer data)
 }
 
 static void
-show_install_dialog (GtkWidget *button, gpointer parent)
+real_show_install_dialog (gpointer parent, gchar *filename)
 {
-	GladeXML *dialog = glade_xml_new (GLADEDIR "/theme-install.glade", NULL, NULL);
-	GtkWidget *widget = WID ("install_dialog");
+	GladeXML *dialog;
+	GtkWidget *widget;
 
+	if (running_theme_install)
+		return;
+
+	running_theme_install = TRUE;
+
+	dialog = glade_xml_new (GLADEDIR "/theme-install.glade", NULL, NULL);
+	widget = WID ("install_dialog");
+	
 	g_signal_connect (G_OBJECT (widget), "response",
 		G_CALLBACK (install_dialog_response), dialog);
 	gtk_window_set_transient_for (GTK_WINDOW (widget), parent);
 	gtk_window_set_position (GTK_WINDOW (widget), GTK_WIN_POS_CENTER_ON_PARENT);
+	if (filename)
+		gnome_file_entry_set_filename (GNOME_FILE_ENTRY (WID ("install_theme_picker")), filename);
 
 	while (gtk_dialog_run (GTK_DIALOG (widget)) == GTK_RESPONSE_HELP)
 		;
 
 	gtk_widget_destroy (widget);
 	g_object_unref (G_OBJECT (dialog));
+
+	running_theme_install = FALSE;
 }
 
+static void
+show_install_dialog (GtkWidget *button, gpointer parent)
+{
+  real_show_install_dialog (parent, NULL);
+}
 /* Callback issued during drag movements */
 
 static gboolean
@@ -392,7 +652,6 @@ drag_data_received_cb (GtkWidget *widget, GdkDragContext *context,
 		       guint info, guint time, gpointer data)
 {
 	GList *uris;
-	GladeXML *dialog = data;
 	gchar *filename;
 
 	if (!(info == TARGET_URI_LIST || info == TARGET_NS_URL))
@@ -406,11 +665,10 @@ drag_data_received_cb (GtkWidget *widget, GdkDragContext *context,
 		g_free (filename);
 		filename = gnome_vfs_uri_to_string (uris->data, GNOME_VFS_URI_HIDE_TOPLEVEL_METHOD);
 	}
-	gnome_file_entry_set_filename (GNOME_FILE_ENTRY (WID ("install_theme_picker")), filename);
-	g_free (filename);
 	gnome_vfs_uri_list_unref (uris);
 
-	show_install_dialog (NULL, widget);
+	real_show_install_dialog (widget, filename);
+	g_free (filename);
 }
 
 static void
@@ -446,39 +704,67 @@ cb_dialog_response (GtkDialog *dialog, gint response_id)
 		gtk_main_quit ();
 }
 
+static void
+setup_tree_view (GtkTreeView *tree_view,
+		 GCallback    changed_callback)
+{
+  GtkTreeModel *model;
+  GtkTreeSelection *selection;
+ 
+  gtk_tree_view_insert_column_with_attributes (tree_view,
+ 					       -1, NULL,
+ 					       gtk_cell_renderer_text_new (),
+ 					       "text", THEME_NAME_COLUMN,
+ 					       NULL);
+  
+  model = (GtkTreeModel *) gtk_list_store_new (N_COLUMNS, G_TYPE_STRING);
+  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model), 0, sort_func, NULL, NULL);
+  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model), 0, GTK_SORT_ASCENDING);
+  gtk_tree_view_set_model (tree_view, model);
+  selection = gtk_tree_view_get_selection (tree_view);
+  gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
+  g_signal_connect (G_OBJECT (selection), "changed", changed_callback, NULL);
+}
+
 static void
 setup_dialog (GladeXML *dialog)
 {
   GConfClient *client;
-  GtkWidget *widget, *parent;
-  GtkTreeModel *model;
-  GtkTreeSelection *selection;
+  GtkWidget *parent, *widget;
 
   client = gconf_client_get_default ();
   parent = WID ("theme_dialog");
 
-  gtk_tree_view_insert_column_with_attributes (GTK_TREE_VIEW (WID ("theme_treeview")),
-					       -1, NULL,
-					       gtk_cell_renderer_text_new (),
-					       "text", THEME_NAME_COLUMN,
-					       NULL);
+  setup_tree_view (GTK_TREE_VIEW (WID ("theme_treeview")),
+		   (GCallback)gtk_theme_selection_changed);
+  setup_tree_view (GTK_TREE_VIEW (WID ("window_theme_treeview")),
+		   (GCallback)window_theme_selection_changed);
+
   gconf_client_add_dir (client, "/desktop/gnome/interface", GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+  gconf_client_add_dir (client, "/desktop/gnome/applications/window_manager", 
+			GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+  gconf_client_add_dir (client, METACITY_THEME_DIR,
+			GCONF_CLIENT_PRELOAD_ONELEVEL, NULL);
+
   gconf_client_notify_add (client,
 			   GTK_THEME_KEY,
 			   (GConfClientNotifyFunc) &theme_key_changed,
 			   dialog, NULL, NULL);
-
-  model = (GtkTreeModel *) gtk_list_store_new (N_COLUMNS, G_TYPE_STRING);
-  gtk_tree_sortable_set_sort_func (GTK_TREE_SORTABLE (model), 0, sort_func, NULL, NULL);
-  gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (model), 0, GTK_SORT_ASCENDING);
-  gtk_tree_view_set_model (GTK_TREE_VIEW (WID ("theme_treeview")), model);
-  selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (WID ("theme_treeview")));
-  gtk_tree_selection_set_mode (selection, GTK_SELECTION_BROWSE);
-  g_signal_connect (G_OBJECT (selection), "changed", (GCallback) theme_selection_changed, NULL);
+  gconf_client_notify_add (client,
+			   WINDOW_THEME_KEY,
+			   (GConfClientNotifyFunc) &window_theme_key_changed,
+			   dialog, NULL, NULL);
+  gconf_client_notify_add (client,
+			   METACITY_THEME_KEY,
+			   (GConfClientNotifyFunc) &metacity_key_changed,
+			   dialog, NULL, NULL);
 
   read_themes (dialog);
+  window_read_themes (dialog);
+
   theme_common_register_theme_change (theme_changed_func, dialog);
 
+  /* app themes */
   widget = WID ("install_button");
   g_signal_connect (G_OBJECT (widget), "clicked",
 		    G_CALLBACK (show_install_dialog), parent);
@@ -486,9 +772,22 @@ setup_dialog (GladeXML *dialog)
   g_signal_connect (G_OBJECT (widget), "clicked",
 		    G_CALLBACK (show_manage_themes), dialog);
 
+  /* window manager themes */
+  widget = WID ("window_install_button");
+  g_signal_connect (G_OBJECT (widget), "clicked",
+		    G_CALLBACK (show_install_dialog), parent);
+  widget = WID ("window_manage_button");
+  g_signal_connect (G_OBJECT (widget), "clicked",
+		    G_CALLBACK (window_show_manage_themes), dialog);
+
+  /*
+  g_signal_connect (G_OBJECT (WID ("install_dialog")), "response",
+		    G_CALLBACK (install_dialog_response), dialog);
+  */
+
   g_signal_connect (G_OBJECT (parent),
-    "response",
-    G_CALLBACK (cb_dialog_response), NULL);
+		    "response",
+		    G_CALLBACK (cb_dialog_response), NULL);
 
   gtk_drag_dest_set (parent, GTK_DEST_DEFAULT_ALL,
 		     drop_types, n_drop_types,
@@ -523,6 +822,8 @@ main (int argc, char *argv[])
   activate_settings_daemon ();
 
   dialog = create_dialog ();
+  gnome_wm_manager_init (WID ("theme_dialog"));
+
   setup_dialog (dialog);
   gtk_main ();
 
