@@ -34,6 +34,7 @@
 #include "xsettings-manager.h"
 #include "gnome-settings-daemon.h"
 
+/*#include "gnome-settings-disk.h"*/
 #include "gnome-settings-font.h"
 #include "gnome-settings-xsettings.h"
 #include "gnome-settings-mouse.h"
@@ -43,6 +44,9 @@
 #include "gnome-settings-wm.h"
 #include "gnome-settings-accessibility-keyboard.h"
 #include "gnome-settings-screensaver.h"
+#include "gnome-settings-default-editor.h"
+#include "gnome-settings-keybindings.h"
+#include "gnome-settings-gtk1theme.h"
 
 #include "GNOME_SettingsDaemon.h"
 
@@ -146,7 +150,9 @@ awake_impl (PortableServer_Servant servant,
 	    const CORBA_char      *service,
 	    CORBA_Environment     *ev)
 {
+#if 0
   printf ("I received an activate request for %s\n", service);
+#endif
   return TRUE;
 }
 
@@ -224,15 +230,19 @@ gnome_settings_daemon_new (void)
    */
   client = gconf_client_get_default ();
 
+/*  gnome_settings_disk_init (client);*/
   gnome_settings_font_init (client);
   gnome_settings_xsettings_init (client);
   gnome_settings_mouse_init (client);
   gnome_settings_keyboard_init (client);
-  gnome_settings_background_init (client);
   gnome_settings_sound_init (client);
   gnome_settings_wm_init (client);
   gnome_settings_accessibility_keyboard_init (client);
   gnome_settings_screensaver_init (client);
+  gnome_settings_default_editor_init (client);
+  gnome_settings_background_init (client);
+  gnome_settings_keybindings_init (client);
+  gnome_settings_gtk1_theme_init (client);
 
   for (list = directories; list; list = list->next)
     {
@@ -261,14 +271,19 @@ gnome_settings_daemon_new (void)
   
   gdk_window_add_filter (NULL, manager_event_filter, NULL);
 
+/*  gnome_settings_disk_load (client);*/
   gnome_settings_font_load (client);
   gnome_settings_xsettings_load (client);
   gnome_settings_mouse_load (client);
+  gnome_settings_keyboard_load (client);
   gnome_settings_sound_load (client);
-  gnome_settings_background_load (client);
   gnome_settings_wm_load (client);
   gnome_settings_accessibility_keyboard_load (client);
   gnome_settings_screensaver_load (client);
-  
+  gnome_settings_default_editor_load (client);
+  gnome_settings_background_load (client);
+  gnome_settings_keybindings_load (client);
+  gnome_settings_gtk1_theme_load (client);
+
   return G_OBJECT (daemon);
 }
